'**************************************************************************************
'                       MJERENJE TEMPERATURE SA VISE DS18B20 SENZORA

'Naziv: Temperature DS18B20 x 2.bas

'Primjer sa 2 senzora DS18B20 na jednom 1-Wire prikljucku.

'Napomena:

'Koriste se tipke UP, ENTER i kratkospojnik JP1

'Koristi se knjizica "multiple_ds18b20.inc" i data tabela "crc8.inc".

'Za offset kalibraciju senzora ne koristi se knjizica "calibrate_multiple_ds.inc".

'Mladen Cempuh, Datum:08.2024.
'**************************************************************************************

$regfile = "m328pdef.dat"                                   ' ATmega328P
$crystal = 16000000                                         ' na 16MHz
$swstack = 50
$hwstack = 40
$framesize = 64

'-----------------------------------------------------------
'DEFINIRANJE LCD DISPLEJA U 4 BITNOM RADU

Config Lcd = 16 * 2                                         ' izabran LCD1602 , za izbor LCD2004 stavi 20 * 4

Config Lcdpin = Pin , Db4 = Portc.3 , Db5 = Portc.2 , Db6 = Portb.5 , Db7 = Portb.4 , Rs = Portc.5 , E = Portc.4
Cls
Cursor Off

Up$ Alias Pind.0                                            ' ulaz za tipku UP je PD.0
Enter$ Alias Pind.2                                         ' ulaz za tipku ENTER je PD.2

Config Up$ = Input                                          ' Up$ i Enter$ su ulazni pinovi mikrokontrolera
Config Enter$ = Input

Portd.0 = 1                                                 ' interni pull up otpornici na ulazima PD.0 i PD.2 ukljuceni
Portd.2 = 1

Backlight Alias Portb.3                                     ' izlaz za upravljanje rasvjetom LCD-a
Config Backlight = Output
Backlight = 1                                               ' rasvjeta ukljucena

Jp1 Alias Pind.3                                            ' Jp1 je ulaz PD.3, to je ulaz za kratkospojnik JP1
Config Jp1 = Input                                          ' to je ulazni pin sa ukljucenim pull up opornikom
Portd.3 = 1

Const Nr_of_ds18b20 = 2                                     ' broj koristenih DS18B20 senzora je 2

'------------------------------------------------------------
$include "multiple_ds18b20.inc"                             ' knjizica za rad vise DS18B20 temperaturnih senzora
'------------------------------------------------------------
Offset_ds_1 Alias 0.15 : Offset_ds_2 Alias -0.05            ' Offset iznosi za kalibraciju koristenih DS18B20 senzora definirani u programu
'------------------------------------------------------------

'Definiraj pin mikrokontrolera za 1-Wire komunikaciju

Config 1wire = Portd.5                                      ' PD.5 je 1-Wire komunikacija za prikljucak DS18B20 senzora
                                                             ' VAZNO: Stavi pull up otpornik 4K7 !!!
'------------------------------------------------------------

If Jp1 = 0 And Up$ = 0 Then                                 ' za kratkospojnik JP1 na ulazu Jp1 i za pritisnutu UP tipku na ulazu Up$
                                                             ' nakon reseta izvrsava se ID identifikacija pojedinacnog DS18B20 senzora
 Ds18b20_identify                                           ' naredba za ID identifikaciju jednog prikljucenog DS18B20 senzora

End If

Do

 Ds18b20                                                    ' naredba za mjerenje temperature, provjeru CRC ispravnosti i izracun temperature
                                                            ' svih DS18B20 senzora

If Err = 1 Then                                             ' Err = 1 --> nema DS18B20 senzora
  Locate 1 , 1 : Lcd "DS18B20 NOT FIND "                    ' ispisi na LCD poruku "DS18B20 NOT FIND"

Else

 If Crc(1) = 0 Then                                         ' Crc(1) = 0 --> senzor DS18B20 br.1 OK
  Temperature_ds(1) = Temperature_ds(1) + Offset_ds_1       ' offset korekcija greske senzora DS18B20 br.1
  Locate 1 , 1 : Lcd "TEMP1 = " ; Fusing(temperature_ds(1) , "#.#") ; Chr(223) ; "C   "       'ispisi temperaturu od DS18B20 br.1 na jednu decimalu
 Else
  Locate 1 , 1 : Lcd "Nr.1 CRC NOT OK!"                     ' CRC za senzor DS18B20 br.1 neispravan, ispisi na LCD poruku "Nr.1 CRC NOT OK!"
 End If

 If Crc(2) = 0 Then                                         ' Crc(2) = 0 --> senzor DS18B20 br.2 OK
  Temperature_ds(2) = Temperature_ds(2) + Offset_ds_2       ' offset korekcija greske senzora DS18B20 br.2
  Locate 2 , 1 : Lcd "TEMP2 = " ; Fusing(temperature_ds(2) , "#.#") ; Chr(223) ; "C   "       'ispisi temperaturu od DS18B20 br.2 na jednu decimalu
 Else
  Locate 2 , 1 : Lcd "Nr.2 CRC NOT OK!"                     ' CRC za senzor DS18B20 br.2 neispravan, ispisi na LCD poruku "Nr.2 CRC NOT OK!"
 End If

End If

Loop

End                                                         ' kraj korisnickog programa

'------------------------------------------------------------
$include "crc8.inc"                                         ' ukljuci data tabelu za brzi izracun CRC8