'**************************************************************************************************
'                             MJERENJE TEMPERATURE SA VISE DS18B20 SENZORA

'Naziv: Temperature DS18B20 x 4 calibrate.bas

'Primjer sa 4 senzora DS18B20 na istom 1-Wire prikljucku sa upotrebom programske knjizice

'"calibrate_multiple_ds.inc" za upis kalibracijskih offset iznosa DS18B20 senzora u interni EEPROM.

'Napomena:

'Koriste se tipke UP, DOWN, ENTER i kratkospojnik JP1

'Koriste se knjizice "multiple_ds18b20.inc" , "calibrate_multiple_ds.inc" i data tabela "crc8.inc".

'Mladen Cempuh, Datum:08.2024.
'**************************************************************************************************

$regfile = "m328pdef.dat"                                   ' ATmega328P
$crystal = 16000000                                         ' na 16MHz
$swstack = 50
$hwstack = 80
$framesize = 64

'-----------------------------------------------------------
'DEFINIRANJE LCD DISPLEJA U 4 BITNOM RADU

Config Lcd = 16 * 2                                         ' izabran LCD1602 , za izbor LCD2004 stavi 20 * 4

Config Lcdpin = Pin , Db4 = Portc.3 , Db5 = Portc.2 , Db6 = Portb.5 , Db7 = Portb.4 , Rs = Portc.5 , E = Portc.4
Cls
Cursor Off

Up$ Alias Pind.0                                            ' ulaz za tipku UP je PD.0
Down$ Alias Pind.1                                          ' ulaz za tipku DOWN je PD.1
Enter$ Alias Pind.2                                         ' ulaz za tipku ENTER je PD.2

Config Up$ = Input                                          ' Up$ , Down$ i Enter$ su ulazni pinovi mikrokontrolera
Config Down$ = Input
Config Enter$ = Input

Portd.0 = 1
Portd.1 = 1                                                 ' interni pull up otpornici na PD.0, PD.1 i PD.2 ukljuceni
Portd.2 = 1

Backlight Alias Portb.3                                     ' izlaz za upravljanje rasvjetom LCD-a
Config Backlight = Output
Backlight = 1                                               ' rasvjeta ukljucena

Jp1 Alias Pind.3                                            ' Jp1 je ulaz PD.3, to je ulaz za kratkospojnik JP1
Config Jp1 = Input                                          ' to je ulazni pin sa ukljucenim pull up opornikom
Portd.3 = 1

Const Nr_of_ds18b20 = 4                                     ' broj koristenih DS18B20 senzora je 4

'------------------------------------------------------------
$include "multiple_ds18b20.inc"                             ' knjizica za rad vise DS18B20 temperaturnih senzora
'------------------------------------------------------------
$include "calibrate_multiple_ds.inc"                        ' knjizica za upis kalibracijskih offset iznosa DS18B20 senzora u EEPROM
'------------------------------------------------------------

'Definiraj pin kontrolera za 1-Wire komunikaciju

Config 1wire = Portd.5                                      ' PD.5 je 1-Wire komunikacija za prikljucak DS18B20 senzora
                                                             ' VAZNO: Stavi pull up otpornik 4K7 !!!
'-------------------------------------------------------------

If Jp1 = 0 And Up$ = 0 Then                                 ' za kratkospojnik JP1 na ulazu Jp1 i za pritisnutu UP tipku na ulazu Up$
                                                             ' nakon reseta izvrsava se ID identifikacija pojedinacnog DS18B20 senzora
 Ds18b20_identify                                           ' naredba za ID identifikaciju jednog prikljucenog DS18B20 senzora

End If

If Jp1 = 0 And Down$ = 0 Then                               ' za kratkospojnik JP1 na ulazu Jp1 i pritisnutu tipku DOWN na ulazu Down$
                                                             ' nakon reseta izvrsava se offset kalibracija DS18B20 senzora
 Calibrate_ds                                               ' naredba za offset kalibraciju DS18B20 senzora

End If


Do

 Ds18b20                                                    ' naredba za mjerenje temperature, provjeru CRC ispravnosti i izracun temperature
                                                            ' svih DS18B20 senzora
Locate 1 , 1

If Err = 1 Then                                             ' Err = 1 --> nema DS18B20 senzora
 Lcd "DS18B20 NOT FIND "                                    ' ispisi na LCD poruku "DS18B20 NOT FIND"

Else

 If Crc(1) = 0 Then                                         ' Crc(1) = 0 --> senzor DS18B20 br.1 OK
  Temperature_ds(1) = Temperature_ds(1) + Offset_ds(1)      ' offset korekcija greske senzora DS18B20 br.1
  Lcd "1:" ; Fusing(temperature_ds(1) , "#.#") ; " "        'ispisi temperaturu od DS18B20 br.1 na jednu decimalu
 Else
  Lcd "1:CRC!  "                                            ' CRC za senzor DS18B20 br.1 neispravan
 End If

 Locate 1 , 9

 If Crc(2) = 0 Then                                         ' Crc(2) = 0 --> senzor DS18B20 br.2 OK
  Temperature_ds(2) = Temperature_ds(2) + Offset_ds(2)      ' offset korekcija greske senzora DS18B20 br.2
  Lcd "2:" ; Fusing(temperature_ds(2) , "#.#") ; " "        'ispisi temperaturu od DS18B20 br.2 na jednu decimalu
 Else
  Lcd "2:CRC!  "                                            ' CRC za senzor DS18B20 br.2 neispravan
 End If

 Locate 2 , 1

 If Crc(3) = 0 Then                                         ' Crc(3) = 0 --> senzor DS18B20 br.3 OK
  Temperature_ds(3) = Temperature_ds(3) + Offset_ds(3)      ' offset korekcija greske senzora DS18B20 br.3
  Lcd "3:" ; Fusing(temperature_ds(3) , "#.#") ; " "        'ispisi temperaturu od DS18B20 br.3 na jednu decimalu
 Else
  Lcd "3:CRC!  "                                            ' CRC za senzor DS18B20 br.3 neispravan
 End If

 Locate 2 , 9

 If Crc(4) = 0 Then                                         ' Crc(4) = 0 --> senzor DS18B20 br.4 OK
  Temperature_ds(4) = Temperature_ds(4) + Offset_ds(4)      ' offset korekcija greske senzora DS18B20 br.4
  Lcd "4:" ; Fusing(temperature_ds(4) , "#.#") ; " "        'ispisi temperaturu od DS18B20 br.4 na jednu decimalu
 Else
  Lcd "4:CRC!  "                                            ' CRC za senzor DS18B20 br.4 neispravan
 End If

End If

Loop

End                                                         ' kraj korisnickog programa

'------------------------------------------------------------
$include "crc8.inc"                                         ' ukljuci data tabelu za brzi izracun CRC8