'*****************************************************************
'      CALIBRATE MORE DS18B20 SENSORS IN ONE CALIBRATION POINT

'Name:"calibrate_multiple_ds.inc"

'Command: Calibrate_ds

'It is used to set the calibration offsets values of several

'DS18B20 sensors and write these values into the internal EEPROM.

'Define key inputs Up$, Down$ and Enter$ in the user program.

'Uses LCD to display data.

'Mladen Cempuh, Datum: 08.2024.
'------------------------------------------------------------------

Dim Offset_ds(nr_of_ds18b20) As Single                      ' array of Single variables in RAM to save the read Offset values from
                                                              ' the internal EEPROM for each DS18B20 sensor in use
'EEPROM variables:

Dim Offset_ds_eeprom(nr_of_ds18b20) As Eram Single          ' array of Single variables in internal EEPROM to save the Offset values
                                                              ' for each used DS18B20 sensor
Dim Offset_marker As Eram Byte                              ' EEPROM marker variable, if<>0 then Offset values was not writable before

If Offset_marker <> 0 Then                                  ' if the eeprom marker variable is not 0 then
 For Probe_nr = 1 To Nr_of_ds18b20
  Offset_ds_eeprom(probe_nr) = 0                            ' write all eeprom offset values to 0
 Next
Else
 For Probe_nr = 1 To Nr_of_ds18b20                          ' the eeprom marker variable is 0 then
 Offset_ds(probe_nr) = Offset_ds_eeprom(probe_nr)           ' read all Offset values from internal EEPROM variables to RAM variables
 Next
End If

Goto End_calibrate_multi_ds                                 ' exit from "calibrate_multi_ds.inc"

'======================= Calibrate_ds ===========================

Sub Calibrate_ds

If Offset_marker <> 0 Then Offset_marker = 0                ' if the Offset_marker in EEPROM is not 0, set it to 0

Probe_nr = 1                                                ' starting ordinal number of DS18B20 sensor is 1

Cls : Lcd "**CALIBRATION**" : Wait 3                        'print a message "**CALIBRATION**" on the LCD

Lab$select_ds:

Cls : Lcd "Select probe ?"

Select_probe_nr_for_calibrate:                              ' selected ordinal number of DS18B20 sensor

Locate 2 , 1 : Lcd "Nr.= " ; Probe_nr

Debounce Up$ , 0 , Probe_nr_incr , Sub                      ' with UP key go to subrutine Probe_nr_incr in "multi_ds18b20.inc"
                                                             ' for increment ordinal number
Debounce Enter$ , 0 , Lab$set_offset                        ' with ENTER key go to enter calibration offset value for selected DS18B20 sensor

Goto Select_probe_nr_for_calibrate

Lab$set_offset:
                                                             ' with Up$ and Down$ keys set the Offset value for the selected DS18B20 sensor
Locate 1 , 1 : Lcd "Probe Nr." ; Probe_nr ; "      "        ' in the range from -2.99 to +2.99 degrees Celsius

Locate 2 , 1 : Lcd "Offset = " ; Fusing(offset_ds(probe_nr) , "#.##") ; Chr(223) ; "C "

If Up$ = 1 And Down$ = 1 Then I$ds = 0

If Up$ = 0 And Offset_ds(probe_nr) < 2.99 Then
 Offset_ds(probe_nr) = Offset_ds(probe_nr) + 0.01
 Gosub Lab$speed$
End If

If Down$ = 0 And Offset_ds(probe_nr) > -2.99 Then
 Offset_ds(probe_nr) = Offset_ds(probe_nr) - 0.01
 Gosub Lab$speed$
End If

Debounce Enter$ , 0 , Lab$write_eeprom_ds                   ' with Enter$ key write the Offset value for selected DS18B20 sensor in EEPROM

Goto Lab$set_offset

Lab$write_eeprom_ds:

Offset_ds_eeprom(probe_nr) = Offset_ds(probe_nr)

Locate 1 , 1 : Lcd "WRITE IN EEPROM!" : Wait 2              ' print message "WRITE IN EEPROM!" on the LCD

Cls : Lcd "END CALIBRATION?"                                ' ask with message "END CALIBRATION?"  and "YES/NO ?"

Tmp$ds = 0

Lab$yes_no_ds:

Locate 2 , 1 : Lcd "YES/NO ? "

If Tmp$ds = 0 Then

 Locate 2 , 10 : Lcd "NO "

Else

 Locate 2 , 10 : Lcd "YES"

End If

If Up$ = 0 Then Tmp$ds = 1                                  ' UP key for YES
If Down$ = 0 Then Tmp$ds = 0                                ' DOWN key for NO

If Enter$ = 0 And Tmp$ds = 1 Then Goto Lab$end_calibrate_ds ' if answer is YES when press Enter$ key end calibration for DS18B20 sensors

If Enter$ = 0 And Tmp$ds = 0 Then Goto Lab$select_ds        ' if answer is NO when press Enter$ key go to select sensor for calibration

Goto Lab$yes_no_ds

Lab$speed$:

If I$ds < 4 Then
 Incr I$ds
 Waitms 800
Else
 Waitms 100
End If

Return

Lab$end_calibrate_ds:

Cls : Lcd "END CALIBRATION!" : Wait 3 : Cls                 ' print message "END CALIBRATION!"  on the LCD

End Sub

End_calibrate_multi_ds:





